/*
  Arduino controller, MQTT-client for ioBroker. Use MQTT Adapter as server in ioBroker (at least 0.2.4)
  IP address: 192.168.69.71
  Platform: Arduino UNO R3 + EthernetShield W5100
  IDE: 1.0.5-r2
  Date: 15.05.2015
  Version: 0.1
  License: MIT, Copyright 2015  electric69
  *****
  It is test sketch. Following analog inputs will be polled: pinA0, pinA1, pinA2. Values have type int (Analog from 0...1023).
  Client ID, and root name "arduinoPLC71".
  Following topics will be created automatically in ioBorker by start:
  1. "arduinoPLC71/status" - status of arduino controller (online, error)
  2. "arduinoPLC71/pollTimer" - send/publish intervall (client is subscribed on this value)
  3. "arduinoPLC71/sensorA0/value" - group sensorA0, measurement result
  4. "arduinoPLC71/sensorA0/status" - group sensorA0, sensor state (ОК, error)
  5. analogically for groups sensorA1 and sensorA2.

  After the controller is started following happens:
  - Set the MAC and static IP of own ethernet driver
  При старте настраивается сетевое соединение (MAC-адрес и статический IP адрес),
  - Init of serial port for output of debug information

  During main loop:
  - If not connected, try to connect to MQTT Broker
  - If connected poll every X seconds the analog ports and publish them after that
  - If the analog port data was read with errors, it (or error code) will be reported in the topic "arduinoPLC71/sensorA0/status" for group sensorA0. Accordingly for other groups.
**/
//------------------------------------------ Libraries ------------------------------------------//
//Use libraries
#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h> //https://github.com/knolleary/pubsubclient
//---------------------------------------- End libraries ----------------------------------------//

//------------------------------------------- Ethernet ------------------------------------------//
//Global constants for network connection: MAC, Own IP, MQTT Broker IP
byte mac[]    = {  0xDE, 0xED, 0xBA, 0xFE, 0xFE, 0x71 };
byte mqttserver[] = { 192, 168, 69, 51 };
byte ip[]     = { 192, 168, 69, 71 };
EthernetClient ethClient;
PubSubClient client(mqttserver, 1883, callback, ethClient);
//----------------------------------------- End ethernet ----------------------------------------//

//------------------------------------------- Sensors -------------------------------------------//
int analogPinA0 = 0;
int analogPinA1 = 1;
int analogPinA2 = 2;
int SensorA0 = 0;
int SensorA1 = 0;
int SensorA2 = 0;
char buff[20];
unsigned long lastTime = 0;
unsigned long poolTime = 5; // Values publish interval: by default 5 seconds
//----------------------------------------- End Sensors -----------------------------------------//

//###############################################################################################//
//########################################## Functions ##########################################//
//###############################################################################################//

//------------------------------------------- Callback -----------------------------------------//
// process publishes from MQTT broker
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.println ("");
  Serial.println ("-------");
  Serial.println ("New callback of MQTT-broker");
  // Convert name(topic) and value (payload) to string
  payload[length] = '\0';
  String strTopic = String(topic);
  String strPayload = String((char*)payload);
  // Ignore "null" values from broker
  if (strPayload == "null") return;
  //Convert string to integer
  poolTime = strPayload.toInt();
  Serial.print (strTopic);
  Serial.print (" ");
  Serial.println (poolTime);
  Serial.println ("-------");
  Serial.println (""); 
}
//----------------------------------------- End callback ---------------------------------------//

//------------------------------------------ Void setup ----------------------------------------//
void setup() {
  Serial.begin(9600);
  Serial.println("Start mqtt-client controller arduinoPLC7...");
  //стартуем сетевое подключение
  Ethernet.begin(mac, ip);
  Serial.print("PLC's IP: ");
  Serial.println(Ethernet.localIP());
}
//---------------------------------------- End void setup --------------------------------------//

//------------------------------------------ Void loop -----------------------------------------//
void loop() {
  client.loop(); //check publishes from broker
  // If no connection
  if (!client.connected()) {
    Serial.print("Connect to MQTT-boker...  ");
    //Try to connect
    if (client.connect("arduinoPLC71")) {
      Serial.println("success");
      client.publish("arduinoPLC71/status","online");
      client.subscribe("arduinoPLC71/pollTimer");
      // ------------------------ CHANGED START -------------------------
      // Force send of values to broker
      lastTime = 0;
      // ------------------------ CHANGED END ---------------------------
    }
    else {
      Serial.println("failed!");
    }
  } else {
    //If connected
    //Read every "poolTime" seconds the values and send to broker
    if(!lastTime || millis() > (lastTime + poolTime*1000)) {
      lastTime = millis();
      Serial.println(" ");
      //read the data and publish it
      SensorA0 = analogRead(analogPinA0);
      sprintf(buff,"%d",SensorA0); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A0 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA0/value",buff);
      client.publish("arduinoPLC71/sensorA0/status","OK");
      SensorA1 = analogRead(analogPinA1);
      sprintf(buff,"%d",SensorA1); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A1 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA1/value",buff);
      client.publish("arduinoPLC71/sensorA1/status","OK");
      SensorA2 = analogRead(analogPinA2);
      sprintf(buff,"%d",SensorA2); //if float dtostrf(SensorA0, 5, 2, buff);
      Serial.print("Publish to MQTT-broker value sensor A2 is: ");
      Serial.println(buff);
      client.publish("arduinoPLC71/sensorA2/value",buff);
      client.publish("arduinoPLC71/sensorA2/status","OK");
      //--------
    }
  }
}
//---------------------------------------- End void loop ---------------------------------------//
